/*
 * tmos - iram.c
 * 	version 0.0.0
 * 
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <asm/iram.h>

static unsigned char iram_brk = 0;

/**
 * __iram_alloc - allocate IRAM memory
 * @size: requested block size
 *
 * The IRAM memory allocator lock must be held.
 */
iram_ptr __iram_alloc(iram_size_t size)
{
	iram_brk -= size;
	return (iram_ptr) iram_brk;
}

/**
 * __iram_free - free IRAM memory
 * @size: block to free
 *
 * @size MUST BE EXACLY the size of block on top of allocation stack.
 * It's illegal to do:
 *
 * __iram_alloc(10);
 * __iram_alloc(20);
 * __iram_free(30);
 *
 * or
 * 
 * __iram_alloc(10);
 * __iram_alloc(20);
 * __iram_free(10);
 * __iram_free(20);
 *
 * The IRAM memory allocator lock must be held.
 */
void __iram_free(iram_size_t size)
{
	iram_brk += size;
}

iram_ptr iram_alloc(iram_size_t size)
{
	iram_ptr ret;

	iram_lock();
	ret = __iram_alloc(size);	
	iram_unlock();
	return ret;
}
