/*
 * tmos - sched.h
 *
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __SCHED_H
#define __SCHED_H

#include <asm/task.h>
#include <setjmp.h>

#define TASK_COMM_LEN	16

struct task {
	struct context __kernel *context;
	pid_t pid;
	char comm[TASK_COMM_LEN];
};

#if 1
extern const struct task __code __task_array[];

#define DECLARE_TASK_POINTER(name) \
	const struct task __code *name

#define for_each_task(task) \
	for (task = __task_array; task->context != KERNEL_NULL; task++)

#define BEGIN_TASK_ARRAY \
	const struct task __code __task_array[] = {

#define TASK(_pid, _comm, _context) \
		{ \
			_context, \
			_pid, \
			_comm, \
		}

#define END_TASK_ARRAY  \
		TASK(1, "", KERNEL_NULL), \
	};
#else
#endif

char task_init(struct task __kernel *task);

#ifdef CONFIG_MULTITASKING
void schedule(void);
void wake_up_context(struct context __kernel *context) __reentrant;
void __remove_run_queue(struct context __kernel *context) __reentrant;

/**
 * __wake_up_context - wake up process from interrupt
 *
 * This function must be called with interrupts disabled
 */
#define __wake_up_context(context)					\
	do {								\
		struct context __kernel *c; 				\
									\
		/* find last task */					\
		for (c = current; c->next != current; c = c->next);	\
									\
		/* add new task after last task */			\
		context->next = current;				\
		c->next = context;					\
	} while (0)
#else
#define schedule() idle()
#endif

#endif
